﻿Imports Microsoft.VisualBasic
Imports System
Imports System.Collections
Imports System.ComponentModel

Namespace GDBG
	Friend Class DataCollection
		Implements IBindingList, INotifyPropertyChanged
		Private list As ArrayList

		Public Sub New()
			list = New ArrayList()
		End Sub

		Default Public Property Item(ByVal index As Integer) As Data
			Get
				Return CType(list(index), Data)
			End Get
			Set(ByVal value As Data)
				list(index) = value
				RaisePropertyChanged(list(index).ToString())
			End Set
		End Property

		Private Sub RaisePropertyChanged(ByVal name As String)
			RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(name))
		End Sub

		#Region "INotifyPropertyChanged Members"

		Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged

		#End Region

		#Region "IBindingList Members"

		Public Sub AddIndex(ByVal [property] As PropertyDescriptor) Implements IBindingList.AddIndex
		End Sub

		Public ReadOnly Property AllowNew() As Boolean Implements IBindingList.AllowNew
			Get
				Return True
			End Get
		End Property

		Public Sub ApplySort(ByVal [property] As PropertyDescriptor, ByVal direction As System.ComponentModel.ListSortDirection) Implements IBindingList.ApplySort
		End Sub

		Public ReadOnly Property SortProperty() As PropertyDescriptor Implements IBindingList.SortProperty
			Get
				Return Nothing
			End Get
		End Property

		Public Function Find(ByVal [property] As PropertyDescriptor, ByVal key As Object) As Integer Implements IBindingList.Find
			Return 0
		End Function

		Public ReadOnly Property SupportsSorting() As Boolean Implements IBindingList.SupportsSorting
			Get
				Return False
			End Get
		End Property

		Public ReadOnly Property IsSorted() As Boolean Implements IBindingList.IsSorted
			Get
				Return False
			End Get
		End Property

		Public ReadOnly Property AllowRemove() As Boolean Implements IBindingList.AllowRemove
			Get
				Return False
			End Get
		End Property

		Public ReadOnly Property SupportsSearching() As Boolean Implements IBindingList.SupportsSearching
			Get
				Return False
			End Get
		End Property

		Public ReadOnly Property SortDirection() As System.ComponentModel.ListSortDirection Implements IBindingList.SortDirection
			Get
				Return New System.ComponentModel.ListSortDirection()
			End Get
		End Property
		Public Event ListChanged As System.ComponentModel.ListChangedEventHandler Implements IBindingList.ListChanged
		Public ReadOnly Property SupportsChangeNotification() As Boolean Implements IBindingList.SupportsChangeNotification
			Get
				Return True
			End Get
		End Property

		Public Sub RemoveSort() Implements IBindingList.RemoveSort
		End Sub

		Public Function AddNew() As Object Implements IBindingList.AddNew
			Dim info As New Data()
			Me.list.Add(info)
			Return info
		End Function

		Public ReadOnly Property AllowEdit() As Boolean Implements IBindingList.AllowEdit
			Get
				Return True
			End Get
		End Property

		Public Sub RemoveIndex(ByVal [property] As PropertyDescriptor) Implements IBindingList.RemoveIndex
		End Sub

		#End Region

		#Region "IList Members"

		Public ReadOnly Property IsReadOnly() As Boolean Implements System.Collections.IList.IsReadOnly
			Get
				Return False
			End Get
		End Property

		Public Property IList_Item(ByVal index As Integer) As Object Implements IList.Item
			Get
				Return list(index)
			End Get
			Set(ByVal value As Object)
				list(index) = value
			End Set
		End Property

		Public Sub RemoveAt(ByVal index As Integer) Implements System.Collections.IList.RemoveAt
			list.RemoveAt(index)
			RaiseListChanged(ListChangedType.ItemDeleted, index)
		End Sub

		Public Sub Insert(ByVal index As Integer, ByVal value As Object) Implements System.Collections.IList.Insert
			list.Insert(index, value)
			RaiseListChanged(ListChangedType.ItemAdded, index)
		End Sub

		Public Sub Remove(ByVal value As Object) Implements System.Collections.IList.Remove
			Dim index As Integer = list.IndexOf(value)
			list.Remove(value)
			RaiseListChanged(ListChangedType.ItemDeleted, index)
		End Sub

		Public Function Contains(ByVal value As Object) As Boolean Implements System.Collections.IList.Contains
			Return list.Contains(value)
		End Function

		Public Sub Clear() Implements System.Collections.IList.Clear
			list.Clear()
		End Sub

		Public Function IndexOf(ByVal value As Object) As Integer Implements System.Collections.IList.IndexOf
			Return list.IndexOf(value)
		End Function

		Public Function Add(ByVal value As Object) As Integer Implements System.Collections.IList.Add
			Dim count As Integer = list.Add(value)
			RaiseListChanged(ListChangedType.ItemAdded, list.Count - 1)
			Return count
		End Function

		Public ReadOnly Property IsFixedSize() As Boolean Implements System.Collections.IList.IsFixedSize
			Get
				Return False
			End Get
		End Property

		Private Sub RaiseListChanged(ByVal type As ListChangedType, ByVal index As Integer)
			RaiseEvent ListChanged(Me, New ListChangedEventArgs(type, index))
		End Sub

		#End Region


		#Region "ICollection Members"

		Public ReadOnly Property IsSynchronized() As Boolean Implements System.Collections.ICollection.IsSynchronized
			Get
				Return False
			End Get
		End Property

		Public ReadOnly Property Count() As Integer Implements System.Collections.ICollection.Count
			Get
				Return list.Count
			End Get
		End Property

		Public Sub CopyTo(ByVal array As Array, ByVal index As Integer) Implements System.Collections.ICollection.CopyTo
			list.CopyTo(array, index)
		End Sub

		Public ReadOnly Property SyncRoot() As Object Implements System.Collections.ICollection.SyncRoot
			Get
				Return list.SyncRoot
			End Get
		End Property

		#End Region

		#Region "IEnumerable Members"

		Public Function GetEnumerator() As IEnumerator Implements System.Collections.IEnumerable.GetEnumerator
			Return list.GetEnumerator()
		End Function

		#End Region
	End Class
End Namespace